@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2021
+# © Copyright EnterpriseDB UK Limited 2018-2022
 #
 # This file is part of Barman.
 #
@@ -20,17 +20,17 @@ import json
 import logging
 from contextlib import closing
 
-import barman
+from barman.clients.cloud_cli import (
+    create_argument_parser,
+    GeneralErrorExit,
+    NetworkErrorExit,
+    OperationErrorExit,
+)
 from barman.cloud import CloudBackupCatalog, configure_logging
 from barman.cloud_providers import get_cloud_interface
 from barman.infofile import BackupInfo
 from barman.utils import force_str
 
-try:
-    import argparse
-except ImportError:
-    raise SystemExit("Missing required python module: argparse")
-
 
 def main(args=None):
     """
@@ -51,29 +51,38 @@ def main(args=None):
             )
 
             if not cloud_interface.test_connectivity():
-                raise SystemExit(1)
+                raise NetworkErrorExit()
             # If test is requested, just exit after connectivity test
             elif config.test:
                 raise SystemExit(0)
 
             if not cloud_interface.bucket_exists:
                 logging.error("Bucket %s does not exist", cloud_interface.bucket_name)
-                raise SystemExit(1)
+                raise OperationErrorExit()
 
             backup_list = catalog.get_backup_list()
 
             # Output
             if config.format == "console":
-                COLUMNS = "{:<20}{:<25}{:<30}"
-                print(COLUMNS.format("Backup ID", "End Time", "Begin Wal"))
+                COLUMNS = "{:<20}{:<25}{:<30}{:<16}"
+                print(
+                    COLUMNS.format(
+                        "Backup ID", "End Time", "Begin Wal", "Archival Status"
+                    )
+                )
                 for backup_id in sorted(backup_list):
                     item = backup_list[backup_id]
                     if item and item.status == BackupInfo.DONE:
+                        keep_target = catalog.get_keep_target(item.backup_id)
+                        keep_status = (
+                            keep_target and "KEEP:%s" % keep_target.upper() or ""
+                        )
                         print(
                             COLUMNS.format(
                                 item.backup_id,
                                 item.end_time.strftime("%Y-%m-%d %H:%M:%S"),
                                 item.begin_wal,
+                                keep_status,
                             )
                         )
             else:
@@ -91,7 +100,7 @@ def main(args=None):
     except Exception as exc:
         logging.error("Barman cloud backup list exception: %s", force_str(exc))
         logging.debug("Exception details:", exc_info=exc)
-        raise SystemExit(1)
+        raise GeneralErrorExit()
 
 
 def parse_arguments(args=None):
@@ -101,70 +110,17 @@ def parse_arguments(args=None):
     :return: The options parsed
     """
 
-    parser = argparse.ArgumentParser(
+    parser, _, _ = create_argument_parser(
         description="This script can be used to list backups "
         "made with barman-cloud-backup command. "
         "Currently AWS S3 and Azure Blob Storage are supported.",
-        add_help=False,
     )
 
-    parser.add_argument(
-        "source_url",
-        help="URL of the cloud source, such as a bucket in AWS S3."
-        " For example: `s3://bucket/path/to/folder`.",
-    )
-    parser.add_argument(
-        "server_name", help="the name of the server as configured in Barman."
-    )
-    parser.add_argument(
-        "-V", "--version", action="version", version="%%(prog)s %s" % barman.__version__
-    )
-    parser.add_argument("--help", action="help", help="show this help message and exit")
-    verbosity = parser.add_mutually_exclusive_group()
-    verbosity.add_argument(
-        "-v",
-        "--verbose",
-        action="count",
-        default=0,
-        help="increase output verbosity (e.g., -vv is more than -v)",
-    )
-    verbosity.add_argument(
-        "-q",
-        "--quiet",
-        action="count",
-        default=0,
-        help="decrease output verbosity (e.g., -qq is less than -q)",
-    )
-    parser.add_argument(
-        "-t",
-        "--test",
-        help="Test cloud connectivity and exit",
-        action="store_true",
-        default=False,
-    )
     parser.add_argument(
         "--format",
         default="console",
         help="Output format (console or json). Default console.",
     )
-    parser.add_argument(
-        "--cloud-provider",
-        help="The cloud provider to use as a storage backend",
-        choices=["aws-s3", "azure-blob-storage"],
-        default="aws-s3",
-    )
-    s3_arguments = parser.add_argument_group(
-        "Extra options for the aws-s3 cloud provider"
-    )
-    s3_arguments.add_argument(
-        "--endpoint-url",
-        help="Override default S3 endpoint URL with the given one",
-    )
-    s3_arguments.add_argument(
-        "-P",
-        "--profile",
-        help="profile name (e.g. INI section in AWS credentials file)",
-    )
     return parser.parse_args(args=args)
 
 
