@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2021
+# © Copyright EnterpriseDB UK Limited 2018-2022
 #
 # This file is part of Barman.
 #
@@ -16,26 +16,25 @@
 # You should have received a copy of the GNU General Public License
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>.
 
-import bz2
-import gzip
 import logging
 import os
 import os.path
-import shutil
 from contextlib import closing
-from io import BytesIO
 
-import barman
+from barman.clients.cloud_cli import (
+    add_tag_argument,
+    create_argument_parser,
+    CLIErrorExit,
+    GeneralErrorExit,
+    NetworkErrorExit,
+    UrlArgumentType,
+)
 from barman.cloud import configure_logging
+from barman.clients.cloud_compression import compress
 from barman.cloud_providers import get_cloud_interface
 from barman.exceptions import BarmanException
-from barman.utils import force_str
-from barman.xlog import hash_dir, is_any_xlog_file
-
-try:
-    import argparse
-except ImportError:
-    raise SystemExit("Missing required python module: argparse")
+from barman.utils import check_positive, check_size, force_str
+from barman.xlog import hash_dir, is_any_xlog_file, is_history_file
 
 
 def __is_hook_script():
@@ -77,7 +76,7 @@ def main(args=None):
     # Validate the WAL file name before uploading it
     if not is_any_xlog_file(config.wal_path):
         logging.error("%s is an invalid name for a WAL file" % config.wal_path)
-        raise SystemExit(1)
+        raise CLIErrorExit()
 
     try:
         cloud_interface = get_cloud_interface(config)
@@ -90,7 +89,7 @@ def main(args=None):
             )
 
             if not cloud_interface.test_connectivity():
-                raise SystemExit(1)
+                raise NetworkErrorExit()
             # If test is requested, just exit after connectivity test
             elif config.test:
                 raise SystemExit(0)
@@ -98,11 +97,16 @@ def main(args=None):
             # TODO: Should the setup be optional?
             cloud_interface.setup_bucket()
 
-            uploader.upload_wal(config.wal_path)
+            upload_kwargs = {}
+            if is_history_file(config.wal_path):
+                upload_kwargs["override_tags"] = config.history_tags
+
+            uploader.upload_wal(config.wal_path, **upload_kwargs)
+
     except Exception as exc:
         logging.error("Barman cloud WAL archiver exception: %s", force_str(exc))
         logging.debug("Exception details:", exc_info=exc)
-        raise SystemExit(1)
+        raise GeneralErrorExit()
 
 
 def parse_arguments(args=None):
@@ -112,50 +116,24 @@ def parse_arguments(args=None):
     :return: The options parsed
     """
 
-    parser = argparse.ArgumentParser(
+    parser, s3_arguments, azure_arguments = create_argument_parser(
         description="This script can be used in the `archive_command` "
         "of a PostgreSQL server to ship WAL files to the Cloud. "
         "Currently AWS S3 and Azure Blob Storage are supported.",
-        add_help=False,
-    )
-    parser.add_argument(
-        "destination_url",
-        help="URL of the cloud destination, such as a bucket in AWS S3."
-        " For example: `s3://bucket/path/to/folder`.",
-    )
-    parser.add_argument(
-        "server_name", help="the name of the server as configured in Barman."
+        source_or_destination=UrlArgumentType.destination,
     )
     parser.add_argument(
         "wal_path",
         nargs="?",
-        help="the value of the '%%p' keyword" " (according to 'archive_command').",
+        help="the value of the '%%p' keyword (according to 'archive_command').",
         default=None,
     )
-    parser.add_argument(
-        "-V", "--version", action="version", version="%%(prog)s %s" % barman.__version__
-    )
-    parser.add_argument("--help", action="help", help="show this help message and exit")
-    verbosity = parser.add_mutually_exclusive_group()
-    verbosity.add_argument(
-        "-v",
-        "--verbose",
-        action="count",
-        default=0,
-        help="increase output verbosity (e.g., -vv is more than -v)",
-    )
-    verbosity.add_argument(
-        "-q",
-        "--quiet",
-        action="count",
-        default=0,
-        help="decrease output verbosity (e.g., -qq is less than -q)",
-    )
     compression = parser.add_mutually_exclusive_group()
     compression.add_argument(
         "-z",
         "--gzip",
-        help="gzip-compress the WAL while uploading to the cloud",
+        help="gzip-compress the WAL while uploading to the cloud "
+        "(should not be used with python < 3.2)",
         action="store_const",
         const="gzip",
         dest="compression",
@@ -163,35 +141,29 @@ def parse_arguments(args=None):
     compression.add_argument(
         "-j",
         "--bzip2",
-        help="bzip2-compress the WAL while uploading to the cloud",
+        help="bzip2-compress the WAL while uploading to the cloud "
+        "(should not be used with python < 3.3)",
         action="store_const",
         const="bzip2",
         dest="compression",
     )
-    parser.add_argument(
-        "-t",
-        "--test",
-        help="Test cloud connectivity and exit",
-        action="store_true",
-        default=False,
+    compression.add_argument(
+        "--snappy",
+        help="snappy-compress the WAL while uploading to the cloud "
+        "(requires optional python-snappy library)",
+        action="store_const",
+        const="snappy",
+        dest="compression",
     )
-    parser.add_argument(
-        "--cloud-provider",
-        help="The cloud provider to use as a storage backend",
-        choices=["aws-s3", "azure-blob-storage"],
-        default="aws-s3",
+    add_tag_argument(
+        parser,
+        name="tags",
+        help="Tags to be added to archived WAL files in cloud storage",
     )
-    s3_arguments = parser.add_argument_group(
-        "Extra options for the aws-s3 cloud provider"
-    )
-    s3_arguments.add_argument(
-        "--endpoint-url",
-        help="Override default S3 endpoint URL with the given one",
-    )
-    s3_arguments.add_argument(
-        "-P",
-        "--profile",
-        help="profile name (e.g. INI section in AWS credentials file)",
+    add_tag_argument(
+        parser,
+        name="history-tags",
+        help="Tags to be added to archived history files in cloud storage",
     )
     s3_arguments.add_argument(
         "-e",
@@ -201,14 +173,33 @@ def parse_arguments(args=None):
         choices=["AES256", "aws:kms"],
         metavar="ENCRYPTION",
     )
-    azure_arguments = parser.add_argument_group(
-        "Extra options for the azure-blob-storage cloud provider"
-    )
     azure_arguments.add_argument(
         "--encryption-scope",
         help="The name of an encryption scope defined in the Azure Blob Storage "
         "service which is to be used to encrypt the data in Azure",
     )
+    azure_arguments.add_argument(
+        "--max-block-size",
+        help="The chunk size to be used when uploading an object via the "
+        "concurrent chunk method (default: 4MB).",
+        type=check_size,
+        default="4MB",
+    )
+    azure_arguments.add_argument(
+        "--max-concurrency",
+        help="The maximum number of chunks to be uploaded concurrently (default: 1).",
+        type=check_positive,
+        default=1,
+    )
+    azure_arguments.add_argument(
+        "--max-single-put-size",
+        help="Maximum size for which the Azure client will upload an object in a "
+        "single request (default: 64MB). If this is set lower than the PostgreSQL "
+        "WAL segment size after any applied compression then the concurrent chunk "
+        "upload method for WAL archiving will be used.",
+        default="64MB",
+        type=check_size,
+    )
     return parser.parse_args(args=args)
 
 
@@ -231,11 +222,13 @@ class CloudWalUploader(object):
         self.compression = compression
         self.server_name = server_name
 
-    def upload_wal(self, wal_path):
+    def upload_wal(self, wal_path, override_tags=None):
         """
         Upload a WAL file from postgres to cloud storage
 
         :param str wal_path: Full path of the WAL file
+        :param List[tuple] override_tags: List of k,v tuples which should override any
+          tags already defined in the cloud interface
         """
         # Extract the WAL file
         wal_name = self.retrieve_wal_name(wal_path)
@@ -252,7 +245,9 @@ class CloudWalUploader(object):
 
         # Put the file in the correct bucket.
         # The put method will handle automatically multipart upload
-        self.cloud_interface.upload_fileobj(fileobj=file_object, key=destination)
+        self.cloud_interface.upload_fileobj(
+            fileobj=file_object, key=destination, override_tags=override_tags
+        )
 
     def retrieve_file_obj(self, wal_path):
         """
@@ -279,23 +274,7 @@ class CloudWalUploader(object):
         if not self.compression:
             return wal_file
 
-        if self.compression == "gzip":
-            # Create a BytesIO for in memory compression
-            in_mem_gzip = BytesIO()
-            # TODO: closing is redundant with python >= 2.7
-            with closing(gzip.GzipFile(fileobj=in_mem_gzip, mode="wb")) as gz:
-                # copy the gzipped data in memory
-                shutil.copyfileobj(wal_file, gz)
-            in_mem_gzip.seek(0)
-            return in_mem_gzip
-
-        elif self.compression == "bzip2":
-            # Create a BytesIO for in memory compression
-            in_mem_bz2 = BytesIO(bz2.compress(wal_file.read()))
-            in_mem_bz2.seek(0)
-            return in_mem_bz2
-        else:
-            raise ValueError("Unknown compression type: %s" % self.compression)
+        return compress(wal_file, self.compression)
 
     def retrieve_wal_name(self, wal_path):
         """
@@ -322,6 +301,10 @@ class CloudWalUploader(object):
         elif self.compression == "bzip2":
             # add bz2 extension
             return "%s.bz2" % wal_name
+
+        elif self.compression == "snappy":
+            # add snappy extension
+            return "%s.snappy" % wal_name
         else:
             raise ValueError("Unknown compression type: %s" % self.compression)
 
