@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2021
+# © Copyright EnterpriseDB UK Limited 2018-2022
 #
 # This file is part of Barman.
 #
@@ -18,19 +18,22 @@
 
 import logging
 import os
+import sys
 from contextlib import closing
 
-import barman
-from barman.cloud import configure_logging
+from barman.clients.cloud_cli import (
+    create_argument_parser,
+    CLIErrorExit,
+    GeneralErrorExit,
+    NetworkErrorExit,
+    OperationErrorExit,
+)
+from barman.cloud import configure_logging, ALLOWED_COMPRESSIONS
 from barman.cloud_providers import get_cloud_interface
+from barman.exceptions import BarmanException
 from barman.utils import force_str
 from barman.xlog import hash_dir, is_any_xlog_file, is_backup_file
 
-try:
-    import argparse
-except ImportError:
-    raise SystemExit("Missing required python module: argparse")
-
 
 def main(args=None):
     """
@@ -45,7 +48,7 @@ def main(args=None):
     # Validate the WAL file name before downloading it
     if not is_any_xlog_file(config.wal_name):
         logging.error("%s is an invalid name for a WAL file" % config.wal_name)
-        raise SystemExit(1)
+        raise CLIErrorExit()
 
     try:
         cloud_interface = get_cloud_interface(config)
@@ -56,21 +59,21 @@ def main(args=None):
             )
 
             if not cloud_interface.test_connectivity():
-                raise SystemExit(1)
+                raise NetworkErrorExit()
             # If test is requested, just exit after connectivity test
             elif config.test:
                 raise SystemExit(0)
 
             if not cloud_interface.bucket_exists:
                 logging.error("Bucket %s does not exist", cloud_interface.bucket_name)
-                raise SystemExit(1)
+                raise OperationErrorExit()
 
             downloader.download_wal(config.wal_name, config.wal_dest)
 
     except Exception as exc:
         logging.error("Barman cloud WAL restore exception: %s", force_str(exc))
         logging.debug("Exception details:", exc_info=exc)
-        raise SystemExit(1)
+        raise GeneralErrorExit()
 
 
 def parse_arguments(args=None):
@@ -80,73 +83,20 @@ def parse_arguments(args=None):
     :return: The options parsed
     """
 
-    parser = argparse.ArgumentParser(
+    parser, _, _ = create_argument_parser(
         description="This script can be used as a `restore_command` "
         "to download WAL files previously archived with "
         "barman-cloud-wal-archive command. "
         "Currently AWS S3 and Azure Blob Storage are supported.",
-        add_help=False,
     )
 
-    parser.add_argument(
-        "source_url",
-        help="URL of the cloud source, such as a bucket in AWS S3."
-        " For example: `s3://bucket/path/to/folder`.",
-    )
-    parser.add_argument(
-        "server_name", help="the name of the server as configured in Barman."
-    )
     parser.add_argument(
         "wal_name",
-        help="The value of the '%%f' keyword " "(according to 'restore_command').",
+        help="The value of the '%%f' keyword (according to 'restore_command').",
     )
     parser.add_argument(
         "wal_dest",
-        help="The value of the '%%p' keyword " "(according to 'restore_command').",
-    )
-    parser.add_argument(
-        "-V", "--version", action="version", version="%%(prog)s %s" % barman.__version__
-    )
-    parser.add_argument("--help", action="help", help="show this help message and exit")
-    verbosity = parser.add_mutually_exclusive_group()
-    verbosity.add_argument(
-        "-v",
-        "--verbose",
-        action="count",
-        default=0,
-        help="increase output verbosity (e.g., -vv is more than -v)",
-    )
-    verbosity.add_argument(
-        "-q",
-        "--quiet",
-        action="count",
-        default=0,
-        help="decrease output verbosity (e.g., -qq is less than -q)",
-    )
-    parser.add_argument(
-        "-t",
-        "--test",
-        help="Test cloud connectivity and exit",
-        action="store_true",
-        default=False,
-    )
-    parser.add_argument(
-        "--cloud-provider",
-        help="The cloud provider to use as a storage backend",
-        choices=["aws-s3", "azure-blob-storage"],
-        default="aws-s3",
-    )
-    s3_arguments = parser.add_argument_group(
-        "Extra options for the aws-s3 cloud provider"
-    )
-    s3_arguments.add_argument(
-        "--endpoint-url",
-        help="Override default S3 endpoint URL with the given one",
-    )
-    s3_arguments.add_argument(
-        "-P",
-        "--profile",
-        help="profile name (e.g. INI section in AWS credentials file)",
+        help="The value of the '%%p' keyword (according to 'restore_command').",
     )
     return parser.parse_args(args=args)
 
@@ -156,9 +106,6 @@ class CloudWalDownloader(object):
     Cloud storage download client
     """
 
-    # Allowed compression algorithms
-    ALLOWED_COMPRESSIONS = {".gz": "gzip", ".bz2": "bzip2"}
-
     def __init__(self, cloud_interface, server_name):
         """
         Object responsible for handling interactions with cloud storage
@@ -200,7 +147,7 @@ class CloudWalDownloader(object):
             elif item.startswith(wal_path):
                 # Detect compression
                 basename = item
-                for e, c in self.ALLOWED_COMPRESSIONS.items():
+                for e, c in ALLOWED_COMPRESSIONS.items():
                     if item[-len(e) :] == e:
                         # Strip extension
                         basename = basename[: -len(e)]
@@ -230,7 +177,13 @@ class CloudWalDownloader(object):
             logging.info(
                 "WAL file %s for server %s does not exists", wal_name, self.server_name
             )
-            raise SystemExit(1)
+            raise OperationErrorExit()
+
+        if compression and sys.version_info < (3, 0, 0):
+            raise BarmanException(
+                "Compressed WALs cannot be restored with Python 2.x - "
+                "please upgrade to a supported version of Python 3"
+            )
 
         # Download the file
         logging.debug(
