@@ -6,7 +6,7 @@
 #
 # See the help page for usage information.
 #
-# © Copyright EnterpriseDB UK Limited 2019-2021
+# © Copyright EnterpriseDB UK Limited 2019-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 from __future__ import print_function
 
+import argparse
 import copy
 import hashlib
 import os
@@ -35,11 +36,6 @@ from io import BytesIO
 
 import barman
 
-try:
-    import argparse
-except ImportError:
-    raise SystemExit("Missing required python module: argparse")
-
 DEFAULT_USER = "barman"
 BUFSIZE = 16 * 1024
 
@@ -132,7 +128,7 @@ def connectivity_test(config):
     ssh_command = build_ssh_command(config)
     try:
         output = subprocess.Popen(ssh_command, stdout=subprocess.PIPE).communicate()
-        print(output[0])
+        print(output[0].decode("utf-8"))
         sys.exit(0)
     except subprocess.CalledProcessError as e:
         exit_with_error("Impossible to invoke remote put-wal: %s" % e)
@@ -184,12 +180,12 @@ def parse_arguments(args=None):
     parser.add_argument(
         "server_name",
         metavar="SERVER_NAME",
-        help="The server name configured in Barman " "from which WALs are taken.",
+        help="The server name configured in Barman from which WALs are taken.",
     )
     parser.add_argument(
         "wal_path",
         metavar="WAL_PATH",
-        help="The value of the '%%p' keyword " "(according to 'archive_command').",
+        help="The value of the '%%p' keyword (according to 'archive_command').",
     )
     return parser.parse_args(args=args)
 
