@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2021
+# © Copyright EnterpriseDB UK Limited 2018-2022
 #
 # This file is part of Barman.
 #
@@ -30,10 +30,13 @@ import signal
 import tarfile
 from abc import ABCMeta, abstractmethod, abstractproperty
 from functools import partial
-from io import BytesIO
+from io import BytesIO, RawIOBase
 from tempfile import NamedTemporaryFile
 
+from barman.annotations import KeepManagerMixinCloud
 from barman.backup_executor import ConcurrentBackupStrategy, ExclusiveBackupStrategy
+from barman.clients import cloud_compression
+from barman.exceptions import BarmanException
 from barman.fs import path_allowed
 from barman.infofile import BackupInfo
 from barman.postgres_plumbing import EXCLUDE_LIST, PGDATA_EXCLUDE_LIST
@@ -45,6 +48,7 @@ from barman.utils import (
     total_seconds,
     with_metaclass,
 )
+from barman import xlog
 
 try:
     # Python 3.x
@@ -57,6 +61,9 @@ except ImportError:
 BUFSIZE = 16 * 1024
 LOGGING_FORMAT = "%(asctime)s [%(process)s] %(levelname)s: %(message)s"
 
+# Allowed compression algorithms
+ALLOWED_COMPRESSIONS = {".gz": "gzip", ".bz2": "bzip2", ".snappy": "snappy"}
+
 
 def configure_logging(config):
     """
@@ -98,7 +105,14 @@ def copyfileobj_pad_truncate(src, dst, length=None):
             dst.write(tarfile.NUL * (remainder - len(buf)))
 
 
-class CloudUploadingError(Exception):
+class CloudProviderError(BarmanException):
+    """
+    This exception is raised when we get an error in the response from the
+    cloud provider
+    """
+
+
+class CloudUploadingError(BarmanException):
     """
     This exception is raised when there are upload errors
     """
@@ -165,8 +179,19 @@ class CloudTarUploader(object):
             self.chunk_size = max(chunk_size, cloud_interface.MIN_CHUNK_SIZE)
         self.buffer = None
         self.counter = 0
-        tar_mode = "w|%s" % (compression or "")
-        self.tar = TarFileIgnoringTruncate.open(fileobj=self, mode=tar_mode)
+        self.compressor = None
+        # Some supported compressions (e.g. snappy) require CloudTarUploader to apply
+        # compression manually rather than relying on the tar file.
+        self.compressor = cloud_compression.get_compressor(compression)
+        # If the compression is supported by tar then it will be added to the filemode
+        # passed to tar_mode.
+        tar_mode = cloud_compression.get_streaming_tar_mode("w", compression)
+        # The value of 65536 for the chunk size is based on comments in the python-snappy
+        # library which suggest it should be good for almost every scenario.
+        # See: https://github.com/andrix/python-snappy/blob/0.6.0/snappy/snappy.py#L282
+        self.tar = TarFileIgnoringTruncate.open(
+            fileobj=self, mode=tar_mode, bufsize=64 << 10
+        )
         self.size = 0
         self.stats = None
 
@@ -175,8 +200,17 @@ class CloudTarUploader(object):
             self.flush()
         if not self.buffer:
             self.buffer = self._buffer()
-        self.buffer.write(buf)
-        self.size += len(buf)
+        if self.compressor:
+            # If we have a custom compressor we must use it here
+            compressed_buf = self.compressor.add_chunk(buf)
+            self.buffer.write(compressed_buf)
+            self.size += len(compressed_buf)
+        else:
+            # If there is no custom compressor then we are either not using
+            # compression or tar has already compressed it - in either case we
+            # just write the data to the buffer
+            self.buffer.write(buf)
+            self.size += len(buf)
 
     def flush(self):
         if not self.upload_metadata:
@@ -262,6 +296,8 @@ class CloudUploadController(object):
             components.append(".gz")
         elif self.compression == "bz2":
             components.append(".bz2")
+        elif self.compression == "snappy":
+            components.append(".snappy")
         return "".join(components)
 
     def _get_tar(self, name):
@@ -460,6 +496,91 @@ class FileUploadStatistics(dict):
         part["start_time"] = start_time
 
 
+class DecompressingStreamingIO(RawIOBase):
+    """
+    Provide an IOBase interface which decompresses streaming cloud responses.
+
+    This is intended to wrap azure_blob_storage.StreamingBlobIO and
+    aws_s3.StreamingBodyIO objects, transparently decompressing chunks while
+    continuing to expose them via the read method of the IOBase interface.
+
+    This allows TarFile to stream the uncompressed data directly from the cloud
+    provider responses without requiring it to know anything about the compression.
+    """
+
+    # The value of 65536 for the chunk size is based on comments in the python-snappy
+    # library which suggest it should be good for almost every scenario.
+    # See: https://github.com/andrix/python-snappy/blob/0.6.0/snappy/snappy.py#L300
+    COMPRESSED_CHUNK_SIZE = 65536
+
+    def __init__(self, streaming_response, decompressor):
+        """
+        Create a new DecompressingStreamingIO object.
+
+        A DecompressingStreamingIO object will be created which reads compressed
+        bytes from streaming_response and decompresses them with the supplied
+        decompressor.
+
+        :param RawIOBase streaming_response: A file-like object which provides the
+          data in the response streamed from the cloud provider.
+        :param barman.clients.cloud_compression.ChunkedCompressor: A ChunkedCompressor
+          object which provides a decompress(bytes) method to return the decompressed bytes.
+        """
+        self.streaming_response = streaming_response
+        self.decompressor = decompressor
+        self.buffer = bytes()
+
+    def _read_from_uncompressed_buffer(self, n):
+        """
+        Read up to n bytes from the local buffer of uncompressed data.
+
+        Removes up to n bytes from the local buffer and returns them. If n is
+        greater than the length of the buffer then the entire buffer content is
+        returned and the buffer is emptied.
+
+        :param int n: The number of bytes to read
+        :return: The bytes read from the local buffer
+        :rtype: bytes
+        """
+        if n <= len(self.buffer):
+            return_bytes = self.buffer[:n]
+            self.buffer = self.buffer[n:]
+            return return_bytes
+        else:
+            return_bytes = self.buffer
+            self.buffer = bytes()
+            return return_bytes
+
+    def read(self, n=-1):
+        """
+        Read up to n bytes of uncompressed data from the wrapped IOBase.
+
+        Bytes are initially read from the local buffer of uncompressed data. If more
+        bytes are required then chunks of COMPRESSED_CHUNK_SIZE are read from the
+        wrapped IOBase and decompressed in memory until >= n uncompressed bytes have
+        been read. n bytes are then returned with any remaining bytes being stored
+        in the local buffer for future requests.
+
+        :param int n: The number of uncompressed bytes required
+        :return: Up to n uncompressed bytes from the wrapped IOBase
+        :rtype: bytes
+        """
+        uncompressed_bytes = self._read_from_uncompressed_buffer(n)
+        if len(uncompressed_bytes) == n:
+            return uncompressed_bytes
+
+        while len(uncompressed_bytes) < n:
+            compressed_bytes = self.streaming_response.read(self.COMPRESSED_CHUNK_SIZE)
+            uncompressed_bytes += self.decompressor.decompress(compressed_bytes)
+            if len(compressed_bytes) < self.COMPRESSED_CHUNK_SIZE:
+                # If we got fewer bytes than we asked for then we're done
+                break
+
+        return_bytes = uncompressed_bytes[:n]
+        self.buffer = uncompressed_bytes[n:]
+        return return_bytes
+
+
 class CloudInterface(with_metaclass(ABCMeta)):
     """
     Abstract base class which provides the interface between barman and cloud
@@ -512,15 +633,18 @@ class CloudInterface(with_metaclass(ABCMeta)):
         """
         pass
 
-    def __init__(self, url, jobs=2):
+    def __init__(self, url, jobs=2, tags=None):
         """
         Base constructor
 
         :param str url: url for the cloud storage resource
         :param int jobs: How many sub-processes to use for asynchronous
           uploading, defaults to 2.
+        :param List[tuple] tags: List of tags as k,v tuples to be added to all
+          uploaded objects
         """
         self.url = url
+        self.tags = tags
 
         # The worker process and the shared queue are created only when
         # needed
@@ -669,7 +793,7 @@ class CloudInterface(with_metaclass(ABCMeta)):
             except KeyboardInterrupt:
                 if not self.abort_requested:
                     logging.info(
-                        "Got abort request: upload cancelled " "(worker %s)",
+                        "Got abort request: upload cancelled (worker %s)",
                         process_number,
                     )
                     self.abort_requested = True
@@ -861,8 +985,8 @@ class CloudInterface(with_metaclass(ABCMeta)):
         """
         extension = os.path.splitext(key)[-1]
         compression = "" if extension == ".tar" else extension[1:]
-        tar_mode = "r|%s" % compression
-        fileobj = self.remote_open(key)
+        tar_mode = cloud_compression.get_streaming_tar_mode("r", compression)
+        fileobj = self.remote_open(key, cloud_compression.get_compressor(compression))
         with tarfile.open(fileobj=fileobj, mode=tar_mode) as tf:
             tf.extractall(path=dst)
 
@@ -921,22 +1045,27 @@ class CloudInterface(with_metaclass(ABCMeta)):
         """
 
     @abstractmethod
-    def remote_open(self, key):
+    def remote_open(self, key, decompressor=None):
         """
         Open a remote object in cloud storage and returns a readable stream
 
         :param str key: The key identifying the object to open
+        :param barman.clients.cloud_compression.ChunkedCompressor decompressor:
+          A ChunkedCompressor object which will be used to decompress chunks of bytes
+          as they are read from the stream
         :return: A file-like object from which the stream can be read or None if
           the key does not exist
         """
 
     @abstractmethod
-    def upload_fileobj(self, fileobj, key):
+    def upload_fileobj(self, fileobj, key, override_tags=None):
         """
         Synchronously upload the content of a file-like object to a cloud key
 
         :param fileobj IOBase: File-like object to upload
         :param str key: The key to identify the uploaded object
+        :param List[tuple] override_tags: List of k,v tuples which should override any
+          tags already defined in the cloud interface
         """
 
     @abstractmethod
@@ -1008,6 +1137,14 @@ class CloudInterface(with_metaclass(ABCMeta)):
         :param str key: The key to use in the cloud service
         """
 
+    @abstractmethod
+    def delete_objects(self, paths):
+        """
+        Delete the objects at the specified paths
+
+        :param List[str] paths:
+        """
+
 
 class CloudBackupUploader(with_metaclass(ABCMeta)):
     """
@@ -1025,7 +1162,11 @@ class CloudBackupUploader(with_metaclass(ABCMeta)):
     """
 
     def __init__(
-        self, server_name, cloud_interface, max_archive_size, compression=None
+        self,
+        server_name,
+        cloud_interface,
+        max_archive_size,
+        compression=None,
     ):
         """
         Base constructor.
@@ -1276,7 +1417,12 @@ class CloudBackupUploaderPostgres(CloudBackupUploader):
     """
 
     def __init__(
-        self, server_name, cloud_interface, max_archive_size, postgres, compression=None
+        self,
+        server_name,
+        cloud_interface,
+        max_archive_size,
+        postgres,
+        compression=None,
     ):
         super(CloudBackupUploaderPostgres, self).__init__(
             server_name,
@@ -1465,7 +1611,7 @@ class BackupFileInfo(object):
         self.additional_files = []
 
 
-class CloudBackupCatalog(object):
+class CloudBackupCatalog(KeepManagerMixinCloud):
     """
     Cloud storage backup catalog
     """
@@ -1478,11 +1624,18 @@ class CloudBackupCatalog(object):
           upload the backup
         :param str server_name: The name of the server as configured in Barman
         """
-
+        super(CloudBackupCatalog, self).__init__(
+            cloud_interface=cloud_interface, server_name=server_name
+        )
         self.cloud_interface = cloud_interface
         self.server_name = server_name
         self.prefix = os.path.join(self.cloud_interface.path, self.server_name, "base")
+        self.wal_prefix = os.path.join(
+            self.cloud_interface.path, self.server_name, "wals"
+        )
         self._backup_list = None
+        self._wal_paths = None
+        self.unreadable_backups = []
 
     def get_backup_list(self):
         """
@@ -1506,6 +1659,7 @@ class CloudBackupCatalog(object):
                     logging.warning(
                         "Unable to open backup.info file for %s: %s" % (backup_id, exc)
                     )
+                    self.unreadable_backups.append(backup_id)
                     continue
 
                 if backup_info:
@@ -1513,6 +1667,50 @@ class CloudBackupCatalog(object):
             self._backup_list = backup_list
         return self._backup_list
 
+    def remove_backup_from_cache(self, backup_id):
+        """
+        Remove backup with backup_id from the cached list. This is intended for
+        cases where we want to update the state without firing lots of requests
+        at the bucket.
+        """
+        if self._backup_list:
+            self._backup_list.pop(backup_id)
+
+    def get_wal_paths(self):
+        """
+        Retrieve a dict of WAL paths keyed by the WAL name from cloud storage
+        """
+        if self._wal_paths is None:
+            wal_paths = {}
+            for wal in self.cloud_interface.list_bucket(
+                self.wal_prefix + "/", delimiter=""
+            ):
+                wal_basename = os.path.basename(wal)
+                if xlog.is_any_xlog_file(wal_basename):
+                    # We have an uncompressed xlog of some kind
+                    wal_paths[wal_basename] = wal
+                else:
+                    # Allow one suffix for compression and try again
+                    wal_name, suffix = os.path.splitext(wal_basename)
+                    if suffix in ALLOWED_COMPRESSIONS and xlog.is_any_xlog_file(
+                        wal_name
+                    ):
+                        wal_paths[wal_name] = wal
+                    else:
+                        # If it still doesn't look like an xlog file, ignore
+                        continue
+
+            self._wal_paths = wal_paths
+        return self._wal_paths
+
+    def remove_wal_from_cache(self, wal_name):
+        """
+        Remove named wal from the cached list. This is intended for cases where
+        we want to update the state without firing lots of requests at the bucket.
+        """
+        if self._wal_paths:
+            self._wal_paths.pop(wal_name)
+
     def get_backup_info(self, backup_id):
         """
         Load a BackupInfo from cloud storage
@@ -1528,11 +1726,14 @@ class CloudBackupCatalog(object):
         backup_info.load(file_object=backup_info_file)
         return backup_info
 
-    def get_backup_files(self, backup_info):
+    def get_backup_files(self, backup_info, allow_missing=False):
         """
         Get the list of expected files part of a backup
 
         :param BackupInfo backup_info: the backup information
+        :param bool allow_missing: True if missing backup files are allowed, False
+         otherwise. A value of False will cause a SystemExit to be raised if any
+         files expected due to the `backup_info` content cannot be found.
         :rtype: dict[int, BackupFileInfo]
         """
         # Correctly format the source path
@@ -1576,6 +1777,8 @@ class CloudBackupCatalog(object):
                         info.compression = "gzip"
                     elif ext == "tar.bz2":
                         info.compression = "bzip2"
+                    elif ext == "tar.snappy":
+                        info.compression = "snappy"
                     else:
                         logging.warning("Skipping unknown extension: %s", ext)
                         continue
@@ -1589,12 +1792,14 @@ class CloudBackupCatalog(object):
                     break
 
         for backup_file in backup_files.values():
+            logging_fun = logging.warning if allow_missing else logging.error
             if backup_file.path is None:
-                logging.error(
+                logging_fun(
                     "Missing file %s.* for server %s",
                     backup_file.base,
                     self.server_name,
                 )
-                raise SystemExit(1)
+                if not allow_missing:
+                    raise SystemExit(1)
 
         return backup_files
