@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2021
+# © Copyright EnterpriseDB UK Limited 2011-2022
 #
 # This file is part of Barman.
 #
@@ -47,19 +47,21 @@ FORBIDDEN_SERVER_NAMES = ["all"]
 
 DEFAULT_USER = "barman"
 DEFAULT_LOG_LEVEL = logging.INFO
-DEFAULT_LOG_FORMAT = "%(asctime)s [%(process)s] %(name)s " "%(levelname)s: %(message)s"
+DEFAULT_LOG_FORMAT = "%(asctime)s [%(process)s] %(name)s %(levelname)s: %(message)s"
 
 _TRUE_RE = re.compile(r"""^(true|t|yes|1|on)$""", re.IGNORECASE)
 _FALSE_RE = re.compile(r"""^(false|f|no|0|off)$""", re.IGNORECASE)
 _TIME_INTERVAL_RE = re.compile(
     r"""
       ^\s*
-      (\d+)\s+(day|month|week)s?  # N (day|month|week) with optional 's'
+      # N (day|month|week|hour) with optional 's'
+      (\d+)\s+(day|month|week|hour)s?
       \s*$
       """,
     re.IGNORECASE | re.VERBOSE,
 )
 _SLOT_NAME_RE = re.compile("^[0-9a-z_]+$")
+_SI_SUFFIX_RE = re.compile(r"""(\d+)\s*(k|Ki|M|Mi|G|Gi|T|Ti)?\s*$""")
 
 REUSE_BACKUP_VALUES = ("copy", "link", "off")
 
@@ -215,6 +217,8 @@ def parse_time_interval(value):
         time_delta = datetime.timedelta(weeks=value)
     elif unit == "m":
         time_delta = datetime.timedelta(days=(31 * value))
+    elif unit == "h":
+        time_delta = datetime.timedelta(hours=value)
     else:
         # This should never happen
         raise ValueError("Invalid unit time %s" % unit)
@@ -222,6 +226,48 @@ def parse_time_interval(value):
     return time_delta
 
 
+def parse_si_suffix(value):
+    """
+    Parse a string, transforming it into integer and multiplying by
+    the SI or IEC suffix
+    eg a suffix of Ki multiplies the integer value by 1024
+    and returns the new value
+
+    Accepted format: N (k|Ki|M|Mi|G|Gi|T|Ti)
+
+    :param str value: the string to evaluate
+    """
+    # if empty string or none return none
+    if value is None or value == "":
+        return None
+    result = _SI_SUFFIX_RE.match(value)
+    if not result:
+        raise ValueError("Invalid value for a number %s" % value)
+    # if the int conversion
+    value = int(result.groups()[0])
+    unit = result.groups()[1]
+
+    # Calculates the value
+    if unit == "k":
+        value *= 1000
+    elif unit == "Ki":
+        value *= 1024
+    elif unit == "M":
+        value *= 1000000
+    elif unit == "Mi":
+        value *= 1048576
+    elif unit == "G":
+        value *= 1000000000
+    elif unit == "Gi":
+        value *= 1073741824
+    elif unit == "T":
+        value *= 1000000000000
+    elif unit == "Ti":
+        value *= 1099511627776
+
+    return value
+
+
 def parse_reuse_backup(value):
     """
     Parse a string to a valid reuse_backup value.
@@ -320,12 +366,16 @@ class ServerConfig(object):
         "conninfo",
         "custom_compression_filter",
         "custom_decompression_filter",
+        "custom_compression_magic",
         "description",
         "disabled",
         "errors_directory",
+        "forward_config_path",
         "immediate_checkpoint",
         "incoming_wals_directory",
         "last_backup_maximum_age",
+        "last_backup_minimum_size",
+        "last_wal_maximum_age",
         "max_incoming_wals_queue",
         "minimum_redundancy",
         "network_compression",
@@ -383,8 +433,12 @@ class ServerConfig(object):
         "configuration_files_directory",
         "custom_compression_filter",
         "custom_decompression_filter",
+        "custom_compression_magic",
+        "forward_config_path",
         "immediate_checkpoint",
         "last_backup_maximum_age",
+        "last_backup_minimum_size",
+        "last_wal_maximum_age",
         "max_incoming_wals_queue",
         "minimum_redundancy",
         "network_compression",
@@ -438,6 +492,7 @@ class ServerConfig(object):
         "check_timeout": "30",
         "disabled": "false",
         "errors_directory": "%(backup_directory)s/errors",
+        "forward_config_path": "false",
         "immediate_checkpoint": "false",
         "incoming_wals_directory": "%(backup_directory)s/incoming",
         "minimum_redundancy": "0",
@@ -470,8 +525,11 @@ class ServerConfig(object):
         "basebackup_retry_times": int,
         "check_timeout": int,
         "disabled": parse_boolean,
+        "forward_config_path": parse_boolean,
         "immediate_checkpoint": parse_boolean,
         "last_backup_maximum_age": parse_time_interval,
+        "last_backup_minimum_size": parse_si_suffix,
+        "last_wal_maximum_age": parse_time_interval,
         "max_incoming_wals_queue": int,
         "network_compression": parse_boolean,
         "parallel_jobs": int,
@@ -518,7 +576,7 @@ class ServerConfig(object):
                     value = parser(new_value)
             except Exception as e:
                 output.warning(
-                    "Ignoring invalid configuration value '%s' " "for key %s in %s: %s",
+                    "Ignoring invalid configuration value '%s' for key %s in %s: %s",
                     new_value,
                     key,
                     source,
