@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2021
+# © Copyright EnterpriseDB UK Limited 2011-2022
 #
 # This file is part of Barman.
 #
@@ -65,7 +65,8 @@ def exec_diagnose(servers, errors_list):
         # server configuration
         diagnosis["servers"][name] = {}
         diagnosis["servers"][name]["config"] = vars(server.config)
-        del diagnosis["servers"][name]["config"]["config"]
+        if "config" in diagnosis["servers"][name]["config"]:
+            del diagnosis["servers"][name]["config"]["config"]
         # server system info
         if server.config.ssh_command:
             try:
@@ -86,4 +87,6 @@ def exec_diagnose(servers, errors_list):
         }
         # Release any PostgreSQL resource
         server.close()
-    output.info(json.dumps(diagnosis, cls=BarmanEncoder, indent=4, sort_keys=True))
+    output.info(
+        json.dumps(diagnosis, cls=BarmanEncoder, indent=4, sort_keys=True), log=False
+    )
