@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2013-2021
+# © Copyright EnterpriseDB UK Limited 2013-2022
 #
 # This file is part of Barman.
 #
@@ -147,6 +147,8 @@ def _put(level, message, *args, **kwargs):
     # handle keyword-only parameters
     log = kwargs.pop("log", True)
     is_error = kwargs.pop("is_error", False)
+    global error_exit_code
+    error_exit_code = kwargs.pop("exit_code", error_exit_code)
     if len(kwargs):
         raise TypeError(
             "%s() got an unexpected keyword argument %r"
@@ -503,7 +505,7 @@ class ConsoleOutputWriter(object):
         if len(results["changes"]) > 0:
             self.info("")
             self.info("IMPORTANT")
-            self.info("These settings have been modified to prevent " "data losses")
+            self.info("These settings have been modified to prevent data losses")
             self.info("")
 
             for assertion in results["changes"]:
@@ -560,7 +562,7 @@ class ConsoleOutputWriter(object):
 
         if results["get_wal"]:
             self.info("")
-            self.info("WARNING: 'get-wal' is in the specified " "'recovery_options'.")
+            self.info("WARNING: 'get-wal' is in the specified 'recovery_options'.")
             self.info(
                 "Before you start up the PostgreSQL server, please "
                 "review the %s file",
@@ -580,11 +582,9 @@ class ConsoleOutputWriter(object):
             ),
         )
         self.info("")
-        self.info(
-            "Your PostgreSQL server has been successfully " "prepared for recovery!"
-        )
+        self.info("Your PostgreSQL server has been successfully prepared for recovery!")
 
-    def _record_check(self, server_name, check, status, hint):
+    def _record_check(self, server_name, check, status, hint, perfdata):
         """
         Record the check line in result_check_map attribute
 
@@ -594,9 +594,16 @@ class ConsoleOutputWriter(object):
         :param str check: the check name
         :param bool status: True if succeeded
         :param str,None hint: hint to print if not None
+        :param str,None perfdata: additional performance data to print if not None
         """
         self.result_check_list.append(
-            dict(server_name=server_name, check=check, status=status, hint=hint)
+            dict(
+                server_name=server_name,
+                check=check,
+                status=status,
+                hint=hint,
+                perfdata=perfdata,
+            )
         )
         if not status and self.active:
             global error_occurred
@@ -620,7 +627,7 @@ class ConsoleOutputWriter(object):
         self.info("Server %s:" % display_name)
         self.active = active
 
-    def result_check(self, server_name, check, status, hint=None):
+    def result_check(self, server_name, check, status, hint=None, perfdata=None):
         """
         Record a server result of a server check
         and output it as INFO
@@ -629,8 +636,10 @@ class ConsoleOutputWriter(object):
         :param str check: the check name
         :param bool status: True if succeeded
         :param str,None hint: hint to print if not None
+        :param str,None perfdata: additional performance data to print if not None
         """
-        self._record_check(server_name, check, status, hint)
+        self._record_check(server_name, check, status, hint, perfdata)
+
         if hint:
             self.info(
                 "\t%s: %s (%s)"
@@ -641,7 +650,7 @@ class ConsoleOutputWriter(object):
 
     def init_list_backup(self, server_name, minimal=False):
         """
-        Init the list-backup command
+        Init the list-backups command
 
         :param str server_name: the server we are start listing
         :param bool minimal: if true output only a list of backup id
@@ -650,7 +659,7 @@ class ConsoleOutputWriter(object):
 
     def result_list_backup(self, backup_info, backup_size, wal_size, retention_status):
         """
-        Output a single backup in the list-backup command
+        Output a single backup in the list-backups command
 
         :param BackupInfo backup_info: backup we are displaying
         :param backup_size: size of base backup (with the required WAL files)
@@ -976,7 +985,7 @@ class ConsoleOutputWriter(object):
                 if getattr(standby, "client_addr", None):
                     self.info("     Communication   : TCP/IP")
                     self.info(
-                        "     IP Address      : %s " "/ Port: %s / Host: %s",
+                        "     IP Address      : %s / Port: %s / Host: %s",
                         standby.client_addr,
                         standby.client_port,
                         standby.client_hostname or "-",
@@ -1021,7 +1030,7 @@ class ConsoleOutputWriter(object):
 
     def init_list_server(self, server_name, minimal=False):
         """
-        Init the list-server command
+        Init the list-servers command
 
         :param str server_name: the server we are start listing
         """
@@ -1029,7 +1038,7 @@ class ConsoleOutputWriter(object):
 
     def result_list_server(self, server_name, description=None):
         """
-        Output a result line of a list-server command
+        Output a result line of a list-servers command
 
         :param str server_name: the server is being checked
         :param str,None description: server description if applicable
@@ -1041,7 +1050,7 @@ class ConsoleOutputWriter(object):
 
     def init_show_server(self, server_name):
         """
-        Init the show-server command output method
+        Init the show-servers command output method
 
         :param str server_name: the server we are displaying
         """
@@ -1049,7 +1058,7 @@ class ConsoleOutputWriter(object):
 
     def result_show_server(self, server_name, server_info):
         """
-        Output the results of the show-server command
+        Output the results of the show-servers command
 
         :param str server_name: the server we are displaying
         :param dict server_info: a dictionary containing the info to display
@@ -1057,6 +1066,22 @@ class ConsoleOutputWriter(object):
         for status, message in sorted(server_info.items()):
             self.info("\t%s: %s", status, message)
 
+    def init_check_wal_archive(self, server_name):
+        """
+        Init the check-wal-archive command output method
+
+        :param str server_name: the server we are displaying
+        """
+        self.info("Server %s:" % server_name)
+
+    def result_check_wal_archive(self, server_name):
+        """
+        Output the results of the check-wal-archive command
+
+        :param str server_name: the server we are displaying
+        """
+        self.info(" - WAL archive check for server %s passed" % server_name)
+
 
 class JsonOutputWriter(ConsoleOutputWriter):
     def __init__(self, *args, **kwargs):
@@ -1215,7 +1240,7 @@ class JsonOutputWriter(ConsoleOutputWriter):
         self.json_output[server_name] = {}
         self.active = active
 
-    def result_check(self, server_name, check, status, hint=None):
+    def result_check(self, server_name, check, status, hint=None, perfdata=None):
         """
         Record a server result of a server check
         and output it as INFO
@@ -1224,8 +1249,9 @@ class JsonOutputWriter(ConsoleOutputWriter):
         :param str check: the check name
         :param bool status: True if succeeded
         :param str,None hint: hint to print if not None
+        :param str,None perfdata: additional performance data to print if not None
         """
-        self._record_check(server_name, check, status, hint)
+        self._record_check(server_name, check, status, hint, perfdata)
         check_key = self._mangle_key(check)
 
         self.json_output[server_name][check_key] = dict(
@@ -1234,7 +1260,7 @@ class JsonOutputWriter(ConsoleOutputWriter):
 
     def init_list_backup(self, server_name, minimal=False):
         """
-        Init the list-backup command
+        Init the list-backups command
 
         :param str server_name: the server we are listing
         :param bool minimal: if true output only a list of backup id
@@ -1244,7 +1270,7 @@ class JsonOutputWriter(ConsoleOutputWriter):
 
     def result_list_backup(self, backup_info, backup_size, wal_size, retention_status):
         """
-        Output a single backup in the list-backup command
+        Output a single backup in the list-backups command
 
         :param BackupInfo backup_info: backup we are displaying
         :param backup_size: size of base backup (with the required WAL files)
@@ -1638,7 +1664,7 @@ class JsonOutputWriter(ConsoleOutputWriter):
 
     def init_list_server(self, server_name, minimal=False):
         """
-        Init the list-server command
+        Init the list-servers command
 
         :param str server_name: the server we are listing
         """
@@ -1647,7 +1673,7 @@ class JsonOutputWriter(ConsoleOutputWriter):
 
     def result_list_server(self, server_name, description=None):
         """
-        Output a result line of a list-server command
+        Output a result line of a list-servers command
 
         :param str server_name: the server is being checked
         :param str,None description: server description if applicable
@@ -1656,7 +1682,7 @@ class JsonOutputWriter(ConsoleOutputWriter):
 
     def init_show_server(self, server_name):
         """
-        Init the show-server command output method
+        Init the show-servers command output method
 
         :param str server_name: the server we are displaying
         """
@@ -1664,7 +1690,7 @@ class JsonOutputWriter(ConsoleOutputWriter):
 
     def result_show_server(self, server_name, server_info):
         """
-        Output the results of the show-server command
+        Output the results of the show-servers command
 
         :param str server_name: the server we are displaying
         :param dict server_info: a dictionary containing the info to display
@@ -1675,6 +1701,24 @@ class JsonOutputWriter(ConsoleOutputWriter):
 
             self.json_output[server_name][status] = message
 
+    def init_check_wal_archive(self, server_name):
+        """
+        Init the check-wal-archive command output method
+
+        :param str server_name: the server we are displaying
+        """
+        self.json_output[server_name] = {}
+
+    def result_check_wal_archive(self, server_name):
+        """
+        Output the results of the check-wal-archive command
+
+        :param str server_name: the server we are displaying
+        """
+        self.json_output[server_name] = (
+            "WAL archive check for server %s passed" % server_name
+        )
+
 
 class NagiosOutputWriter(ConsoleOutputWriter):
     """
@@ -1694,6 +1738,137 @@ class NagiosOutputWriter(ConsoleOutputWriter):
         Do not print anything on standard error
         """
 
+    def _parse_check_results(self):
+        """
+        Parse the check results and return the servers checked and any issues.
+
+        :return tuple: a tuple containing a list of checked servers, a list of all
+            issues found and a list of additional performance detail.
+        """
+        # List of all servers that have been checked
+        servers = []
+        # List of servers reporting issues
+        issues = []
+        # Nagios performance data
+        perf_detail = []
+        for item in self.result_check_list:
+            # Keep track of all the checked servers
+            if item["server_name"] not in servers:
+                servers.append(item["server_name"])
+            # Keep track of the servers with issues
+            if not item["status"] and item["server_name"] not in issues:
+                issues.append(item["server_name"])
+            # Build the performance data list
+            if item["check"] == "backup minimum size":
+                perf_detail.append(
+                    "%s=%dB" % (item["server_name"], int(item["perfdata"]))
+                )
+            if item["check"] == "wal size":
+                perf_detail.append(
+                    "%s_wals=%dB" % (item["server_name"], int(item["perfdata"]))
+                )
+        return servers, issues, perf_detail
+
+    def _summarise_server_issues(self, issues):
+        """
+        Converts the supplied list of issues into a printable summary.
+
+        :return tuple: A tuple where the first element is a string summarising each
+            server with issues and the second element is a string containing the
+            details of all failures for each server.
+        """
+        fail_summary = []
+        details = []
+        for server in issues:
+            # Join all the issues for a server. Output format is in the
+            # form:
+            # "<server_name> FAILED: <failed_check1>, <failed_check2> ... "
+            # All strings will be concatenated into the $SERVICEOUTPUT$
+            # macro of the Nagios output
+            server_fail = "%s FAILED: %s" % (
+                server,
+                ", ".join(
+                    [
+                        item["check"]
+                        for item in self.result_check_list
+                        if item["server_name"] == server and not item["status"]
+                    ]
+                ),
+            )
+            fail_summary.append(server_fail)
+            # Prepare an array with the detailed output for
+            # the $LONGSERVICEOUTPUT$ macro of the Nagios output
+            # line format:
+            # <servername>.<failed_check1>: FAILED
+            # <servername>.<failed_check2>: FAILED (Hint if present)
+            # <servername2.<failed_check1>: FAILED
+            # .....
+            for issue in self.result_check_list:
+                if issue["server_name"] == server and not issue["status"]:
+                    fail_detail = "%s.%s: FAILED" % (server, issue["check"])
+                    if issue["hint"]:
+                        fail_detail += " (%s)" % issue["hint"]
+                    details.append(fail_detail)
+        return fail_summary, details
+
+    def _print_check_failure(self, servers, issues, perf_detail):
+        """Prints the output for a failed check."""
+        # Generate the performance data message - blank string if no perf detail
+        perf_detail_message = perf_detail and "|%s" % " ".join(perf_detail) or ""
+
+        fail_summary, details = self._summarise_server_issues(issues)
+        # Append the summary of failures to the first line of the output
+        # using * as delimiter
+        if len(servers) == 1:
+            print(
+                "BARMAN CRITICAL - server %s has issues * %s%s"
+                % (servers[0], " * ".join(fail_summary), perf_detail_message)
+            )
+        else:
+            print(
+                "BARMAN CRITICAL - %d server out of %d have issues * "
+                "%s%s"
+                % (
+                    len(issues),
+                    len(servers),
+                    " * ".join(fail_summary),
+                    perf_detail_message,
+                )
+            )
+
+        # add the detailed list to the output
+        for issue in details:
+            print(issue)
+
+    def _print_check_success(self, servers, issues=None, perf_detail=None):
+        """Prints the output for a successful check."""
+        if issues is None:
+            issues = []
+
+        # Generate the issues message - blank string if no issues
+        issues_message = "".join([" * IGNORING: %s" % issue for issue in issues])
+        # Generate the performance data message - blank string if no perf detail
+        perf_detail_message = perf_detail and "|%s" % " ".join(perf_detail) or ""
+
+        # Some issues, but only in skipped server
+        good = [item for item in servers if item not in issues]
+        # Display the output message for a single server check
+        if len(good) == 0:
+            print("BARMAN OK - No server configured%s" % issues_message)
+        elif len(good) == 1:
+            print(
+                "BARMAN OK - Ready to serve the Espresso backup "
+                "for %s%s%s" % (good[0], issues_message, perf_detail_message)
+            )
+        else:
+            # Display the output message for several servers, using
+            # '*' as delimiter
+            print(
+                "BARMAN OK - Ready to serve the Espresso backup "
+                "for %d servers * %s%s%s"
+                % (len(good), " * ".join(good), issues_message, perf_detail_message)
+            )
+
     def close(self):
         """
         Display the result of a check run as expected by Nagios.
@@ -1703,17 +1878,7 @@ class NagiosOutputWriter(ConsoleOutputWriter):
 
         global error_occurred, error_exit_code
 
-        # List of all servers that have been checked
-        servers = []
-        # List of servers reporting issues
-        issues = []
-        for item in self.result_check_list:
-            # Keep track of all the checked servers
-            if item["server_name"] not in servers:
-                servers.append(item["server_name"])
-            # Keep track of the servers with issues
-            if not item["status"] and item["server_name"] not in issues:
-                issues.append(item["server_name"])
+        servers, issues, perf_detail = self._parse_check_results()
 
         # Global error (detected at configuration level)
         if len(issues) == 0 and error_occurred:
@@ -1722,94 +1887,10 @@ class NagiosOutputWriter(ConsoleOutputWriter):
             return
 
         if len(issues) > 0 and error_occurred:
-            fail_summary = []
-            details = []
-            for server in issues:
-                # Join all the issues for a server. Output format is in the
-                # form:
-                # "<server_name> FAILED: <failed_check1>, <failed_check2> ... "
-                # All strings will be concatenated into the $SERVICEOUTPUT$
-                # macro of the Nagios output
-                server_fail = "%s FAILED: %s" % (
-                    server,
-                    ", ".join(
-                        [
-                            item["check"]
-                            for item in self.result_check_list
-                            if item["server_name"] == server and not item["status"]
-                        ]
-                    ),
-                )
-                fail_summary.append(server_fail)
-                # Prepare an array with the detailed output for
-                # the $LONGSERVICEOUTPUT$ macro of the Nagios output
-                # line format:
-                # <servername>.<failed_check1>: FAILED
-                # <servername>.<failed_check2>: FAILED (Hint if present)
-                # <servername2.<failed_check1>: FAILED
-                # .....
-                for issue in self.result_check_list:
-                    if issue["server_name"] == server and not issue["status"]:
-                        fail_detail = "%s.%s: FAILED" % (server, issue["check"])
-                        if issue["hint"]:
-                            fail_detail += " (%s)" % issue["hint"]
-                        details.append(fail_detail)
-            # Append the summary of failures to the first line of the output
-            # using * as delimiter
-            if len(servers) == 1:
-                print(
-                    "BARMAN CRITICAL - server %s has issues * %s"
-                    % (servers[0], " * ".join(fail_summary))
-                )
-            else:
-                print(
-                    "BARMAN CRITICAL - %d server out of %d have issues * "
-                    "%s" % (len(issues), len(servers), " * ".join(fail_summary))
-                )
-
-            # add the detailed list to the output
-            for issue in details:
-                print(issue)
+            self._print_check_failure(servers, issues, perf_detail)
             error_exit_code = 2
-        elif len(issues) > 0 and not error_occurred:
-            # Some issues, but only in skipped server
-            good = [item for item in servers if item not in issues]
-            # Display the output message for a single server check
-            if len(good) == 0:
-                print(
-                    "BARMAN OK - No server configured * IGNORING: %s"
-                    % (" * IGNORING: ".join(issues))
-                )
-            elif len(good) == 1:
-                print(
-                    "BARMAN OK - Ready to serve the Espresso backup "
-                    "for %s * IGNORING: %s" % (good[0], " * IGNORING: ".join(issues))
-                )
-            else:
-                # Display the output message for several servers, using
-                # '*' as delimiter
-                print(
-                    "BARMAN OK - Ready to serve the Espresso backup "
-                    "for %d servers * %s * IGNORING: %s"
-                    % (len(good), " * ".join(good), " * IGNORING: ".join(issues))
-                )
         else:
-            # No issues, all good!
-            # Display the output message for a single server check
-            if not len(servers):
-                print("BARMAN OK - No server configured")
-            elif len(servers) == 1:
-                print(
-                    "BARMAN OK - Ready to serve the Espresso backup "
-                    "for %s" % (servers[0])
-                )
-            else:
-                # Display the output message for several servers, using
-                # '*' as delimiter
-                print(
-                    "BARMAN OK - Ready to serve the Espresso backup "
-                    "for %d servers * %s" % (len(servers), " * ".join(servers))
-                )
+            self._print_check_success(servers, issues, perf_detail)
 
 
 #: This dictionary acts as a registry of available OutputWriters
