@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2021
+# © Copyright EnterpriseDB UK Limited 2011-2022
 #
 # This file is part of Barman.
 #
@@ -201,9 +201,7 @@ class RecoveryExecutor(object):
         try:
             recovery_info["cmd"].create_dir_if_not_exists(dest)
         except FsOperationFailed as e:
-            output.error(
-                "unable to initialise destination directory " "'%s': %s", dest, e
-            )
+            output.error("unable to initialise destination directory '%s': %s", dest, e)
             output.close_and_exit()
 
         # Initialize tablespace directories
@@ -324,7 +322,7 @@ class RecoveryExecutor(object):
             recovery_info["cmd"].create_dir_if_not_exists(archive_status_dir)
         except FsOperationFailed as e:
             output.error(
-                "unable to create the archive_status directory " "'%s': %s",
+                "unable to create the archive_status directory '%s': %s",
                 archive_status_dir,
                 e,
             )
@@ -417,7 +415,7 @@ class RecoveryExecutor(object):
                 )
             except FsOperationFailed:
                 output.error(
-                    "Unable to connect to the target host using the command " "'%s'",
+                    "Unable to connect to the target host using the command '%s'",
                     remote_command,
                 )
                 output.close_and_exit()
@@ -565,7 +563,7 @@ class RecoveryExecutor(object):
 
             if target_action:
                 raise RecoveryTargetActionException(
-                    "Can't enable recovery target action when PITR " "is not required"
+                    "Can't enable recovery target action when PITR is not required"
                 )
 
         recovery_info["target_epoch"] = target_epoch
@@ -612,14 +610,14 @@ class RecoveryExecutor(object):
             # on destination directory
             safe_horizon = None
             _logger.warning(
-                "Unable to retrieve safe horizon time " "for smart rsync copy: %s", e
+                "Unable to retrieve safe horizon time for smart rsync copy: %s", e
             )
         except Exception as e:
             # Same as above, but something failed decoding .barman-recover.info
             # or comparing times, so log the full traceback
             safe_horizon = None
             _logger.exception(
-                "Error retrieving safe horizon time " "for smart rsync copy: %s", e
+                "Error retrieving safe horizon time for smart rsync copy: %s", e
             )
 
         recovery_info["safe_horizon"] = safe_horizon
@@ -643,7 +641,7 @@ class RecoveryExecutor(object):
             cmd.create_dir_if_not_exists(tblspc_dir)
         except FsOperationFailed as e:
             output.error(
-                "unable to initialise tablespace directory " "'%s': %s", tblspc_dir, e
+                "unable to initialise tablespace directory '%s': %s", tblspc_dir, e
             )
             output.close_and_exit()
         for item in backup_info.tablespaces:
@@ -672,7 +670,7 @@ class RecoveryExecutor(object):
                 cmd.create_symbolic_link(location, pg_tblspc_file)
             except FsOperationFailed as e:
                 output.error(
-                    "unable to prepare '%s' tablespace " "(destination '%s'): %s",
+                    "unable to prepare '%s' tablespace (destination '%s'): %s",
                     item.name,
                     location,
                     e,
@@ -945,7 +943,7 @@ class RecoveryExecutor(object):
                     ":%s" % os.path.join(recovery_info["wal_dest"], "archive_status"),
                 )
             except CommandFailedException as e:
-                output.error("unable to populate archive_status " "directory: %s", e)
+                output.error("unable to populate archive_status directory: %s", e)
                 output.close_and_exit()
 
     def _generate_recovery_conf(
@@ -1260,6 +1258,7 @@ class RecoveryExecutor(object):
         # Write the mangled content
         mangled = []
         with open(filename, "wb") as f:
+            last_line = None
             for l_number, line in enumerate(content):
                 rm = PG_CONF_SETTING_RE.match(line.decode("utf-8"))
                 if rm:
@@ -1277,10 +1276,12 @@ class RecoveryExecutor(object):
                             )
                         )
                         continue
+                last_line = line
                 f.write(line)
-            # Append content of append_lises array
+            # Append content of append_lines array
             if append_lines:
-                if line[-1] != "\n".encode("utf-8"):
+                # Ensure we have end of line character at the end of the file before adding new lines
+                if last_line and last_line[-1] != "\n".encode("utf-8"):
                     f.write("\n".encode("utf-8"))
                 f.write(("\n".join(append_lines) + "\n").encode("utf-8"))
 
